#include"duff.hpp"




namespace libduff{




Element::
Element()
{
}


Element::
Element(int  i)
{
  reset_integer(i);
}


Element::
Element(uint  i)
{
  reset_integer(i);
}


Element::
Element(const std::string&  s)
{
  reset_string(s);
}


Element::
Element(const void*  ptr, size_t  size)
{
  reset_binary(ptr,size);
}


Element::
Element(const Element&  rhs):
refcounter_(rhs.refcounter_),
data_type_(rhs.data_type_),
general_i(rhs.general_i)
{
    if(rhs.buffer_)
    {
      buffer_.reset(new uint8_t[general_i+1]);

      std::memcpy(buffer_.get(),rhs.buffer_.get(),general_i);

      buffer_[general_i] = 0;
    }
}


Element::
Element(Element&&  rhs):
refcounter_(rhs.refcounter_),
data_type_(rhs.data_type_),
general_i(rhs.general_i),
buffer_(std::move(rhs.buffer_))
{
}




DataType
Element::
get_data_type() const
{
  return data_type_;
}


void
Element::
set_data_type(DataType  data_type)
{
  data_type_ = data_type;
}




int
Element::
get_int() const
{
  return((get_data_type() == DataType::integer)? general_i: -general_i);
}


uint
Element::
get_uint() const
{
  return general_i;
}


void
Element::
reset_integer(int  i)
{
  set_data_type((i >= 0)? DataType::integer:
                          DataType::negative_integer);

  general_i = std::abs(i);


  buffer_.reset();
}


void
Element::
reset_integer(uint  i)
{
  set_data_type(DataType::integer);

  general_i = i;


  buffer_.reset();
}


void
Element::
reset_string(const std::string&  s)
{
  set_data_type(DataType::string);

  general_i = s.size();

  buffer_.reset(new uint8_t[general_i+1]);

  snprintf((char*)buffer_.get(),general_i+1,"%s",s.data());
}


void
Element::
reset_binary(const void*  ptr, size_t  size)
{
  set_data_type(DataType::binary);

  general_i = size;

  buffer_.reset(new uint8_t[general_i]);

  std::memcpy(buffer_.get(),ptr,size);
}




void
Element::
print_indent(int  indent)
{
    while(indent--)
    {
      fputc(' ',stdout);
    }
}


vlintmax_t
Element::
get_general_integer() const
{
  return general_i;
}


const void*
Element::
get_buffer() const
{
  return buffer_.get();
}




bool
Element::
operator==(const Element&  rhs) const
{
    if((data_type_ == rhs.data_type_) &&
       (general_i == rhs.general_i))
    {
      auto  a =     buffer_.get();
      auto  b = rhs.buffer_.get();

        if(!a && !b)
        {
          return true;
        }

      else
        if(a && b && (std::memcmp(a,b,general_i) == 0))
        {
          return true;
        }
    }


  return false;
}


void
Element::
write(Stream&  s) const
{
  put_vlint(s,(int)data_type_);

  put_vlint(s,general_i);

    switch(data_type_)
    {
      case(DataType::string):
      case(DataType::binary):
        s.write(buffer_.get(),general_i);
        break;
    }
}


void
Element::
read(Stream&  s)
{
  data_type_ = (DataType)get_vlint(s);

  general_i = get_vlint(s);

  buffer_.reset();

    switch(data_type_)
    {
      case(DataType::string):
      case(DataType::binary):
        buffer_.reset(new uint8_t[general_i+1]);

        s.read(buffer_.get(),general_i);

        buffer_[general_i] = 0;
        break;
    }
}


void
Element::
print(int  indent) const
{
  print_indent(indent);

  printf("<%s>: ",get_data_type_string(data_type_));

    switch(data_type_)
    {
      case(DataType::integer):
        printf("%" PRIuVL "(%08" PRIXVL")",general_i,general_i);
        break;
      case(DataType::negative_integer):
        printf("%" PRIdVL "(%08" PRIXVL")",-general_i,general_i);
        break;
      case(DataType::string):
        printf("\"%s\"",(const char*)buffer_.get());
        break;
      case(DataType::binary):
        printf("...");
        break;
    }
}




}




