#include"duff.hpp"




namespace libduff{




Node::
Node(Volume&  volume):
volume_(volume)
{
}


Node::Node(const Node&   rhs): volume_(rhs.volume_), refcounter_(rhs.refcounter_), links_(          rhs.links_ ){}
Node::Node(      Node&&  rhs): volume_(rhs.volume_), refcounter_(rhs.refcounter_), links_(std::move(rhs.links_)){}




Node&
Node::
make_node(const char*  name)
{
  auto  n = volume_.new_node();

  links_.emplace(name,Link(LinkKind::node,n));

  return volume_.get_node(n);
}


void
Node::
make_tag(const char*  name)
{
  links_.emplace(name,Link(LinkKind::null));
}


void
Node::
insert_element(const char*  name, Element&&  element)
{
  auto  n = volume_.new_element(std::move(element));

  links_.emplace(name,Link(LinkKind::element,n));
}




void
Node::
renumber(const RenumberingData&  dat)
{
    for(auto&  p: links_)
    {
      auto&  ln = p.second;

      vlintmax_t  n;

        switch(ln.get_kind())
        {
          case(LinkKind::node):
            n = dat.node_numbers_.at(ln.get_target_number());
            ln.set_target_number(n);
            break;
          case(LinkKind::element):
            n = dat.element_numbers_.at(ln.get_target_number());
            ln.set_target_number(n);
            break;
        }
    }
}


bool
Node::
operator==(const Node&  rhs) const
{
  return(links_ == rhs.links_);
}




void
Node::
write(Stream&  s) const
{
  put_vlint(s,links_.size());

    for(auto&  p: links_)
    {
      auto&  name = p.first;

      put_vlint(s,name.size());

      s.write(name.data(),name.size());

      p.second.write(s);
    }
}


void
Node::
read(Stream&  s)
{
  links_.clear();

  auto  count = get_vlint(s);

    while(count--)
    {
      std::string  name;

      auto  n = get_vlint(s);

        while(n--)
        {
          name.push_back(s.get());
        }


      auto  lnptr = find_link(name.data());

        if(lnptr)
        {
          report;

          return;
        }


      links_[name].read(s);
    }
}


void
Node::
print(int  indent) const
{
  Element::print_indent(indent);

  printf("has %d links\n",links_.size());

    for(auto&  p: links_)
    {
      auto&  ln = p.second;

      Element::print_indent(indent);

        if(ln.get_kind() == LinkKind::node)
        {
          auto&  child = volume_.get_node(ln.get_target_number());

          printf("<node>(%s){\n",p.first.data());

          child.print(indent+1);

          printf("}\n");
        }

      else
        if(ln.get_kind() == LinkKind::element)
        {
          printf("<element>(%s){",p.first.data());

          volume_.get_element(ln.get_target_number()).print(indent);

          printf("}\n");
        }

      else
        if(ln.get_kind() == LinkKind::null)
        {
          printf("<>(%s)\n",p.first.data());
        }
    }
}




}




