#ifndef LIBDUFF_NODE_HPP_INCLUDED
#define LIBDUFF_NODE_HPP_INCLUDED


#include"duff_link.hpp"
#include<unordered_map>




namespace libduff{


class Volume;


struct
RenumberingData
{
  std::unordered_map<UniqueNumber,UniqueNumber>     node_numbers_;
  std::unordered_map<UniqueNumber,UniqueNumber>  element_numbers_;

};


class
Node
{
  friend class Volume;

  ReferenceCounter  refcounter_;

  Volume&  volume_;

  std::unordered_map<std::string,Link>  links_;

public:
  Node(Volume&  volume);
  Node(const Node&   rhs);
  Node(      Node&&  rhs);


  void  make_tag(const char*  name);

  Node&  make_node(const char*  name);

  void  insert_element(const char*  name, Element&&  element);

  template<typename...  ARGS> void  make_element(const char*  name, ARGS...  args)
  {
    insert_element(name,Element(args...));
  }


        Link*  find_link(const char*  name)      ;
  const Link*  find_link(const char*  name) const;

        Node*  find_node(const char*  name)      ;
  const Node*  find_node(const char*  name) const;

        Element*  find_element(const char*  name)      ;
  const Element*  find_element(const char*  name) const;

  template<typename  T> void  process(T&  t)
  {
      for(auto&  ln: links_)
      {
        t(ln);
      }
  }


  void  renumber(const RenumberingData&  dat);

  bool  operator==(const Node&  rhs) const;

  void   read(Stream&  s)      ;
  void  write(Stream&  s) const;

  void  print(int  indent) const;

};




}




#endif




