#include"duff_stream.hpp"




namespace{


uint32_t
crc_table[256];


void
make_crc_table()
{
    for(uint32_t  i = 0;  i < 256;  i++)
    {
      uint32_t c = i;

        for(int  j = 0;  j < 8;  j++)
        {
          c = (c & 1) ? (0xEDB88320 ^ (c >> 1)) : (c >> 1);
        }


      crc_table[i] = c;
    }
}


}




namespace libduff{




Stream::
Stream():
crc32_flag_(0)
{
}


Stream::
~Stream()
{
}




size_t
Stream::
get_pos() const
{
  return 0;
}


void
Stream::
set_pos(size_t  pos)
{
}


int
Stream::
get()
{
  return 0;
}


int
Stream::
put(int  c)
{
  auto  uc = static_cast<unsigned char>(c);

    if(crc32_flag_)
    {
      crc32_ = crc_table[(crc32_ ^ uc) & 0xFF] ^ (crc32_ >> 8);
    }


  return uc;
}




void
Stream::
reset_crc32(bool  v)
{
  static bool  initialized;

    if(!initialized)
    {
      make_crc_table();

      initialized = true;
    }


  crc32_ = 0xFFFFFFFF;

  crc32_flag_ = v;
}


uint32_t
Stream::
get_crc32() const
{
  return crc32_ ^ 0xFFFFFFFF;
}




void
Stream::
write(const void*  buf, size_t  bytes)
{
  auto  src = static_cast<const char*>(buf);

    while(bytes--)
    {
      put(*src++);
    }
}


void
Stream::
read(void*  buf, size_t  bytes)
{
  auto  dst = static_cast<char*>(buf);

    while(bytes--)
    {
      *dst++ = get();
    }
}



MemoryStream::
MemoryStream():
unique_s_(new std::string),
s_(unique_s_.get()),
offset_(0)
{
}


MemoryStream::
MemoryStream(std::string&  s):
s_(&s),
offset_(0)
{
}




const std::string&
MemoryStream::
get_buffer() const
{
  return *s_;
}


size_t
MemoryStream::
get_pos() const
{
  return offset_;
}


void
MemoryStream::
set_pos(size_t  pos)
{
  offset_ = pos;
}


int
MemoryStream::
get()
{
  return static_cast<uint8_t>((*s_)[offset_++]);
}


int
MemoryStream::
put(int  c)
{
  c = Stream::put(c);

    if(offset_ < s_->size())
    {
      (*s_)[offset_] = c;
    }

  else
    {
      s_->push_back(c);
    }


  offset_ += 1;

  return c;
}




ConstMemoryStream::
ConstMemoryStream(const std::string&  s):
s_(s),
offset_(0)
{
}


ConstMemoryStream::
~ConstMemoryStream()
{
}




void
ConstMemoryStream::
set_pos(size_t  pos)
{
  offset_ = pos;
}


size_t
ConstMemoryStream::
get_pos() const
{
  return offset_;
}


int
ConstMemoryStream::
get()
{
  return static_cast<uint8_t>(s_[offset_++]);
}




FileStream::
FileStream(FILE*  f): f_(f)
{
}


FileStream::
~FileStream()
{
    if(f_)
    {
      fclose(f_);
    }
}




void
FileStream::
set_pos(size_t  pos)
{
  fseek(f_,pos,SEEK_SET);
}


size_t
FileStream::
get_pos() const
{
  return ftell(f_);
}


int
FileStream::
get()
{
  return fgetc(f_);
}


int
FileStream::
put(int  c)
{
  Stream::put(c);

  return fputc(c,f_);
}




}




