#ifndef LIBDUFF_STREAM_HPP_INCLUDED
#define LIBDUFF_STREAM_HPP_INCLUDED


#include<cstdio>
#include<cstdint>
#include<cstddef>
#include<cstring>
#include<string>
#include<memory>
#include<utility>




namespace libduff{


class
Stream
{
  uint32_t  crc32_;

  int  crc32_flag_;

public:
           Stream();
  virtual ~Stream();

  virtual size_t  get_pos(           ) const;
  virtual void    set_pos(size_t  pos)      ;

  virtual int  get(      );
  virtual int  put(int  c);

  void  reset_crc32(bool  v);

  uint32_t  get_crc32() const;

  void  write(const void*  buf, size_t  bytes);
  void   read(      void*  buf, size_t  bytes);

};




class
MemoryStream: public Stream
{
  std::unique_ptr<std::string>  unique_s_;

  std::string*  s_;

  size_t  offset_;

public:
   MemoryStream(               );
   MemoryStream(std::string&  s);


  const std::string&  get_buffer() const;

  size_t  get_pos(           ) const;
  void    set_pos(size_t  pos)      ;

  int  get();
  int  put(int  c);

};


class
ConstMemoryStream: public Stream
{
  const std::string&  s_;

  size_t  offset_;

public:
   ConstMemoryStream(const std::string&  s);
  ~ConstMemoryStream();

  size_t  get_pos(           ) const;
  void    set_pos(size_t  pos)      ;

  int  get();

};




class
FileStream: public Stream
{
  FILE*  f_;

public:
   FileStream(FILE*  f);
  ~FileStream();

  size_t  get_pos(           ) const;
  void    set_pos(size_t  pos)      ;

  int  get(      );
  int  put(int  c);

};




}




#endif




