#include"duff.hpp"
#include<random>




using namespace libduff;




bool
test_vlint_rw(MemoryStream&  m, vlintmax_t  v)
{
  m.set_pos(0);

  printf("source value size is %d bytes.\n",get_vlint_size(v));

  put_vlint(m,v);

  printf("%d bytes is written.\n",m.get_pos());

  m.set_pos(0);

  vlintmax_t  res = get_vlint(m);

  printf("%d bytes is read.\n",m.get_pos());

  printf("source value is 0x%08" PRIXVL ".\n",v);
  printf("result value is 0x%08" PRIXVL ".\n\n",res);

  return(v == res);
}


void
test_vlint()
{
  bool  result = true;

  printf("****  vlint sequential number test  ****\n");


  MemoryStream  m;

  vlintmax_t  v = 0;

    for(;;)
    {
        if(!test_vlint_rw(m,v))
        {
          printf("test failed.\n");

          result = false;

          break;
        }


        if(v >= VLINT_MAX)
        {
          break;
        }


      v <<= 1;
      v  |= 1;
    }


  printf("****  vlint random number test  ****\n");

  std::random_device  seed;

  std::default_random_engine  engine(seed());

  v = 0x7F;

    for(;;)
    {
      std::uniform_int_distribution<vlintmax_t>  dist(0,v);

        for(int  i = 0;  i < 40;  ++i)
        {
            if(!test_vlint_rw(m,dist(engine)))
            {
              printf("test failed.\n");

              result = false;

              break;
            }
        }


        if(v >= VLINT_MAX)
        {
          break;
        }


      v <<= 1;
      v  |= 1;
    }


    if(result)
    {
      printf("\nall tests passed.\n\n");
    }
}


void
test_volume_rw(FILE*  fptr)
{
  FileStream  f(fptr);

  Volume  a_vol;
  Volume  b_vol;

  vol.read(f);

  vol.print();
}


int
main(int  argc, char**  argv)
{
  FILE*  f = (argv[1]? fopen(argv[1],"rb"):nullptr);

    if(f)
    {
      test_volume_rw(f);

      fclose(f);
    }

  else
    {
      test_vlint();
    }


  return 0;
}




