#include"duff.hpp"




namespace libduff{


/*
1byte ,  7bits 1xxxxxxx

2bytes, 12bits 0LLLxxxx xxxxxxxx
3bytes, 20bits 0LLLxxxx xxxxxxxx xxxxxxxx
4bytes, 28bits 0LLLxxxx xxxxxxxx xxxxxxxx xxxxxxxx
5bytes, 36bits 0LLL0000 xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx

//6bytes, 44bits 0LLLxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx
//7bytes, 52bits 0LLLxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx
//8bytes, 60bits 0LLLxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx
//9bytes, 64bits 0LLL0000 xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx


Lの部分3ビットが表す値に2を足したものが、可変長整数のバイト数
*/


size_t
get_vlint_size(vlintmax_t  i)
{
  size_t  size;

       if(i <= 0x0000007F){size = 1;}
  else if(i <= 0x00000FFF){size = 2;}
  else if(i <= 0x000FFFFF){size = 3;}
  else if(i <= 0x0FFFFFFF){size = 4;}
  else if(i <= 0xFFFFFFFF){size = 5;}

  return size;
}


void
put_vlint(Stream&  s, vlintmax_t  i)
{
    if(i <= 0x7F)
    {
      s.put(i|0x80);

      return;
    }


    switch(get_vlint_size(i))
    {
      case(2):
        s.put(i>>8);
        s.put(i&0xFF);
        break;
      case(3):
        s.put((1<<4)|(i>>16));
        s.put((i>>8)&0xFF);
        s.put((i>>0)&0xFF);
        break;
      case(4):
        s.put((2<<4)|(i>>24));
        s.put((i>>16)&0xFF);
        s.put((i>> 8)&0xFF);
        s.put((i>> 0)&0xFF);
        break;
      case(5):
        s.put(3<<4);
        s.put((i>>24)&0xFF);
        s.put((i>>16)&0xFF);
        s.put((i>> 8)&0xFF);
        s.put((i>> 0)&0xFF);
        break;
    }
}


vlintmax_t
get_vlint(Stream&  s)
{
  uint8_t  first_byte = s.get();

    if(first_byte&0x80)
    {
      return(first_byte&0x7F);
    }


  size_t  size = (first_byte>>4);

  vlintmax_t  i;

    switch(size)
    {
      case(0):
        i = ((first_byte&0x0F)<<8);
        i |= s.get();
        break;
      case(1):
        i = ((first_byte&0x0F)<<16);
        i |= (s.get()<<8);
        i |= (s.get()   );
        break;
      case(2):
        i = ((first_byte&0x0F)<<24);
        i |= (s.get()<<16);
        i |= (s.get()<< 8);
        i |= (s.get()    );
        break;
      case(3):
        i = 0;//((first_byte&0x0F)<<32);
        i |= (s.get()<<24);
        i |= (s.get()<<16);
        i |= (s.get()<< 8);
        i |= (s.get()    );
        break;
    }


  return i;
}


}




