#include"duff.hpp"




namespace libduff{




Volume::
Volume(const char*  name):
name_(name),
last_node_number_(0),
last_element_number_(0)
{
  new_node();
}


Volume::
Volume(Volume&&  rhs):
name_(std::move(rhs.name_)),
last_node_number_(rhs.last_node_number_),
last_element_number_(rhs.last_element_number_),
nodes_(std::move(rhs.nodes_)),
elements_(std::move(rhs.elements_))
{
}




Node&
Volume::
get_node(UniqueNumber  n)
{
  return nodes_.at(n);
}


const Node&
Volume::
get_node(UniqueNumber  n) const
{
  return nodes_.at(n);
}




Element&
Volume::
get_element(UniqueNumber  n)
{
  return elements_.at(n);
}


const Element&
Volume::
get_element(UniqueNumber  n) const
{
  return elements_.at(n);
}




UniqueNumber
Volume::
new_node()
{
  auto  n = last_node_number_++;

  nodes_.emplace(n,Node(*this));

  return n;
}


void
Volume::
delete_node(UniqueNumber  target_number)
{
    if(target_number)
    {
      nodes_.erase(target_number);
    }
}




UniqueNumber
Volume::
new_element()
{
  auto  n = last_element_number_++;

  elements_.emplace(n,Element());

  return n;
}


UniqueNumber
Volume::
new_element(Element&&  element)
{
  auto  n = last_element_number_++;

  elements_.emplace(n,std::move(element));

  return n;
}


void
Volume::
delete_element(UniqueNumber  target_number)
{
  elements_.erase(target_number);
}




void
Volume::
refer_node(UniqueNumber  target_number)
{
  get_node(target_number).refcounter_.value_ += 1;
}


void
Volume::
unrefer_node(UniqueNumber  target_number)
{
  auto&  v = get_node(target_number).refcounter_.value_;

    if(target_number && !--v)
    {
      delete_node(target_number);
    }
}


void
Volume::
refer_element(UniqueNumber  target_number)
{
  get_element(target_number).refcounter_.value_ += 1;
}


void
Volume::
unrefer_element(UniqueNumber  target_number)
{
  auto&  v = get_element(target_number).refcounter_.value_;

    if(!--v)
    {
      delete_element(target_number);
    }
}




bool
Volume::
operator==(const Volume&  rhs) const
{
  return((name_ == rhs.name_) &&
         (last_node_number_ == rhs.last_node_number_) &&
         (last_element_number_ == rhs.last_element_number_) &&
         (get_node(0) == rhs.get_node(0)));
}




enum{
   volume_version = 0,
     node_version = 0,
  element_version = 0,
};


void
Volume::
write(Stream&  s) const
{
  s.put(volume_version);


  put_vlint(s,name_.size());

  s.write(name_.data(),name_.size());


  s.put(node_version);

  put_vlint(s,nodes_.size());

    for(auto&  p: nodes_)
    {
      put_vlint(s,p.first);

      p.second.write(s);
    }


  s.put(element_version);

  put_vlint(s,elements_.size());

    for(auto&  p: elements_)
    {
      put_vlint(s,p.first);

      p.second.write(s);
    }
}


void
Volume::
read(Stream&  s)
{
      name_.clear();
     nodes_.clear();
  elements_.clear();

  last_node_number_    = 0;
  last_element_number_ = 0;

  const int  volver = s.get();

  size_t  size = get_vlint(s);

    while(size--)
    {
      name_.push_back(s.get());
    }


  RenumberingData  renumdat;

  const int  nodver = s.get();

  auto  count = get_vlint(s);

    while(count--)
    {
      auto  old_n = get_vlint(s);
      auto  new_n =   new_node();

      renumdat.node_numbers_[old_n] = new_n;

      get_node(new_n).read(s);
    }


  const int  elmver = s.get();

  count = get_vlint(s);

    while(count--)
    {
      auto  old_n = get_vlint(s);
      auto  new_n = new_element();

      renumdat.element_numbers_[old_n] = new_n;

      get_element(new_n).read(s);
    }


    for(auto&  ndsh: nodes_)
    {
      ndsh.second.renumber(renumdat);
    }
}


void
Volume::
print() const
{
  printf("volume: %s\n",name_.data());
  printf("has %d nodes\n",nodes_.size());
  printf("has %d elements\n",elements_.size());

  get_node(0).print(0);

  printf("\n\n");
}




}





